@@ -1,7 +1,7 @@
 use v6;
 use fatal;
 
-unit class Getopt::Long:ver<0.1.2>;
+unit class Getopt::Long:ver<0.1.3>;
 
 my sub null-converter(Str:D $value --> Str) {
 	return $value;
@@ -90,7 +90,7 @@ has Option:D %!options;
 submethod BUILD(:%!options) { }
 
 method !options {
-	return %!options;
+	return %!options.values;
 }
 
 my %store-for = (
@@ -184,6 +184,16 @@ my grammar Argument {
 	}
 }
 
+role Exceptional {
+}
+
+class Exception is ::Exception does Exceptional {
+	has $.message;
+	method new(Str $message) {
+		return self.bless(:$message);
+	}
+}
+
 method new-from-patterns(@patterns, *%args) {
 	my %options;
 	for @patterns -> $pattern {
@@ -193,7 +203,7 @@ method new-from-patterns(@patterns, *%args) {
 			}
 		}
 		else {
-			die "Couldn't parse '$pattern'";
+			die Exception.new("Couldn't parse '$pattern'");
 		}
 	}
 	return self.new(|%args, :%options);
@@ -210,23 +220,22 @@ my %converter-for-type{Any:U} = (
 );
 
 my role Formatted {
-	has Str:D $.format is required;
+	has Positional $.format is required;
 }
 
-multi sub trait_mod:<is>(Parameter $param, Str:D :getopt($format)!) is export(:DEFAULT, :traits) {
-	$param does Formatted(:$format);
+multi sub trait_mod:<is>(Parameter $param, Str:D :$getopt!) is export(:DEFAULT, :traits) {
+	with Argument.parse($getopt, :rule('argument')) -> $match {
+		return $param does Formatted(:format($match.ast));
+	}
+	else {
+		die Exception.new("Couldn't parse '$getopt'");
+	}
 }
 
 my sub parse-parameter(Parameter $param) {
 	my @names = $param.named_names;
 	if $param ~~ Formatted {
-		my $pattern = $param.format;
-		if Argument.parse($pattern, :rule('argument')) -> $match {
-			return make-option(@names, |@($match.ast))
-		}
-		else {
-			die "Couldn't parse '$pattern'";
-		}
+		return make-option(@names, |$param.format.list);
 	}
 	else {
 		if $param.sigil eq '$' {
@@ -256,21 +265,17 @@ multi sub trait_mod:<is>(Sub $sub, :$getopt!) is export(:DEFAULT, :traits) {
 }
 
 method new-from-sub(Sub $main) {
-	return $main.getopt if $main ~~ Parsed;
-
-	my sub get-options($candidate) {
-		if $candidate ~~ Parsed {
-			return $candidate.getopt!options.values;
-		}
-		else {
-			return $candidate.signature.params.grep(*.named).map(&parse-parameter).flat;
-		}
+	my multi get-options($candidate) {
+		return $candidate.signature.params.grep(*.named).map(&parse-parameter).flat;
+	}
+	my multi get-options(Parsed $candidate) {
+		return $candidate.getopt!options;
 	}
 	my %options;
 	for $main.candidates -> $candidate {
 		for get-options($candidate) -> $option {
 			if %options{$option.name}:exists and %options{$option.name} !eqv $option {
-				die "Can't merge arguments for {$option.name}";
+				die Exception.new("Can't merge arguments for {$option.name}");
 			}
 			%options{$option.name} = $option;
 		}
@@ -286,7 +291,7 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 		my $consumed = 0;
 
 		sub get-option($key) {
-			return %!options{$key} // die "Unknown option $key";
+			return %!options{$key} // die Exception.new("Unknown option $key");
 		}
 
 		sub take-value($option, $value) {
@@ -295,7 +300,7 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 
 			CATCH {
 				when X::Str::Numeric {
-					$_ does role {
+					$_ does role :: does Exceptional {
 						method message() {
 							qq{Cannot convert --$option.name() argument "$value" to number: $.reason};
 						}
@@ -303,7 +308,7 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 					.rethrow;
 				}
 				when X::Numeric::CannotConvert {
-					$_ does role {
+					$_ does role :: does Exceptional {
 						method message() {
 							"Cannot convert --{$option.name} argument $.source to {$.target // $.target.perl}: $.reason"
 						}
@@ -325,7 +330,7 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 				$option.store-default(%hash);
 			}
 			elsif $consumed < $option.arity.min {
-				die "No argument given for option {$option.name}";
+				die Exception.new("No argument given for option {$option.name}");
 			}
 		}
 
@@ -351,7 +356,7 @@ method get-options(@args is copy, :%hash, :named-anywhere(:$permute) = False, :$
 		}
 		elsif $head ~~ / ^ '--' $<name>=[<[\w-]>+] '=' $<value>=[.*] / -> $/ {
 			my $option = get-option($<name>);
-			die  "$<name> doesn't take arguments" if $option.arity.max == 0;
+			die Exception.new("$<name> doesn't take arguments") if $option.arity.max == 0;
 			take-value($option, ~$<value>);
 			take-args($option);
 		}
@@ -391,7 +396,7 @@ our sub get-options-from(@args, *@elements, :$overwrite, *%config) is export(:DE
 			@options.push: $key;
 		}
 		default {
-			die "Unknown element type: " ~ $element.perl;
+			die Exception.new("Unknown element type: " ~ $element.perl);
 		}
 	}
 	my $getopt = Getopt::Long.new-from-patterns(@options);
